/********************************************************************************
 *
 *	FILE NAME	:	clg3_io.c
 *	CREATE		:	2007.10.26
 *	MCU			:	M32/C 85
 *	VERSION		:	1.00
 *	DESCRIPTION	:	
 *
 *==history======================================================================
 *
 ********************************************************************************/

/********************************************************************************
 *		Include Section															*
 ********************************************************************************/
#include "clg3_io.h"

/********************************************************************************
 *		Prot-Type Section														*
 ********************************************************************************/
unsigned char clg3InitIo(void);
unsigned char clg3InputIo(void);
unsigned char clg3GetPush(unsigned char,unsigned char);
unsigned char clg3GetDip(unsigned char sts);
unsigned char clg3SetDig(unsigned char,unsigned char,unsigned char);
unsigned char clg3DispDig(void);

/********************************************************************************
 *		Structure Section														*
 ********************************************************************************/
/* SFRp\ */
typedef union{
	struct{
		unsigned char b0:1;
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;
	}bit;
	unsigned char byte;
}byte_def;

/********************************************************************************
 *		SFR Section																*
 ********************************************************************************/
#pragma ADDRESS		p2_addr		03e4H	/* |[g2WX^ */
#pragma ADDRESS		pd2_addr	03e6H	/* |[g2WX^ */
#pragma ADDRESS		p3_addr		03e5H	/* |[g3WX^ */
#pragma ADDRESS		pd3_addr	03e7H	/* |[g3WX^ */
#pragma ADDRESS		p4_addr		03e8H	/* |[g4WX^ */
#pragma ADDRESS		pd4_addr	03eaH	/* |[g4WX^ */
#pragma ADDRESS		p10_addr	03c8H	/* |[g10WX^ */
#pragma ADDRESS		pd10_addr	03caH	/* |[g10WX^ */

/* |[g` */
byte_def p2_addr;
byte_def p3_addr;
byte_def p4_addr;
byte_def p10_addr;
/* WX^` */
byte_def pd2_addr;
byte_def pd3_addr;
byte_def pd4_addr;
byte_def pd10_addr;

#define p2		p2_addr.byte
#define p3		p3_addr.byte
#define p4		p4_addr.byte
#define p10		p10_addr.byte
#define pd2		pd2_addr.byte
#define pd4		pd4_addr.byte
#define pd3		pd3_addr.byte
#define pd10	pd10_addr.byte

/********************************************************************************
 *		definition Section														*
 ********************************************************************************/
#define TRUE		(1)
#define FALSE		(0)

#define DIP_SW		(p2)
#define PUSH_SW		(p10)
#define SEL_SEG		(p3)
#define DIG			(p4)

#define DIG_CLR_MSK  (0x0F)		/* DIGI𖳂 */
#define DIG1_SEL_MSK (0xFE)		/* DIG1I */
#define DIG2_SEL_MSK (0xFD)		/* DIG2I */
#define DIG3_SEL_MSK (0xFB)		/* DIG3I */
#define DIG4_SEL_MSK (0xF7)		/* DIG4I */

/********************************************************************************
 *		Variable Section														*
 ********************************************************************************/
unsigned char sts_dip;
unsigned char sts_push[3];
unsigned char dig_data[4];
unsigned char set_seg;

const unsigned char CLG3IO_DIG_TBL[30] = {
	 0x3F	/* 0011 1111	: 0 */
	,0x06	/* 0000 0110	: 1 */
	,0x5B	/* 0101 1011	: 2 */
	,0x4F	/* 0100 1111	: 3 */
	,0x66	/* 0110 0110	: 4 */
	,0x6D	/* 0110 1101	: 5 */
	,0x7D	/* 0111 1101	: 6 */
	,0x07	/* 0000 0111	: 7 */
	,0x7F	/* 0111 1111	: 8 */
	,0x6F	/* 0110 1111	: 9 */
	,0x77	/* 0111 0111	: A */
	,0x7C	/* 0111 1100	: B */
	,0x39	/* 0011 1001	: C */
	,0x5E	/* 0101 1110	: D */
	,0x79	/* 0111 1001	: E */
	,0x71	/* 0111 0001	: F */
	,0x3D	/* 0011 1101	: G */
	,0x76	/* 0111 0110	: H */
	,0x30	/* 0011 0000	: I */
	,0x1E	/* 0001 1110	: J */
	,0x38	/* 0011 1000	: L */
	,0x5C	/* 0101 1100	: o */
	,0x73	/* 0111 0011	: P */
	,0x50	/* 0110 0000	: r */
	,0x6D	/* 0110 1101	: S */
	,0x78	/* 0111 1000	: t */
	,0x3E	/* 0011 1110	: U */
	,0x40	/* 0100 0000	: - */
	,0x80	/* 1000 0000	: . */
	,0x00	/* 0000 0000	:   */
};

const unsigned char CLG3_DISP_TBL[4] = {
	 DIG1_SEL_MSK
	,DIG2_SEL_MSK
	,DIG3_SEL_MSK
	,DIG4_SEL_MSK
};

/********************************************************************************
 *
 *	Function	:	unsigned char clg3InitIo(void)
 *	INPUT		:	None
 *	OUTPUT		:	FALSE	:	s
 *					TRUE	:	
 *	DESCRIPTION	:
 *
 *==history======================================================================
 *
 ********************************************************************************/
unsigned char clg3InitIo(void){
	/* auto variable */
	unsigned char result = TRUE;
	
	/*  */
	pd2  = 0x00;	/* DIP */
	pd3  = 0x0f;	/* DIGI */
	pd4  = 0xff;	/* DIGo */
	pd10 = 0x00;	/* PSH */
	
	/*  */
	SEL_SEG	= 0xff;	/* I */
	
	sts_push[0] = 0x00;
	sts_push[1] = 0x00;
	sts_push[2] = 0x00;
	dig_data[0] = 0x00;
	dig_data[1] = 0x00;
	dig_data[2] = 0x00;
	dig_data[3] = 0x00;
	set_seg     = 0x00;

	
	if(pd2 != 0x00 || pd3 != 0x0f || pd4 != 0xff || pd10 != 0xf0){
		result = FALSE;
	}
	
	return result;
}

/********************************************************************************
 *
 *	Function	:	unsigned char clg3InputIo(void)
 *	INPUT		:	None
 *	OUTPUT		:	FALSE	:	s
 *					TRUE	:	
 *	DESCRIPTION	:
 *
 *==history======================================================================
 *
 ********************************************************************************/
unsigned char clg3InputIo(void){
	/* auto variable */
	unsigned char result = TRUE;
	
	/* Olۑ */
	sts_push[0] = sts_push[1];
	
	/* lۑ */
	sts_dip = ~DIP_SW;
	sts_push[1] = ~PUSH_SW;	/* Low Active */
	
	/* gK(PUSH SŴ) */
	sts_push[2] = (sts_push[1] & (~sts_push[0]));
	
	return result;
}

/********************************************************************************
 *
 *	Function	:	unsigned char clg3GetPush(unsigned char sw_data,unsigned char sts)
 *	INPUT		:	sw_data	:	擾SW
 *					sts		:	擾SW
 *	OUTPUT		:	result	:	SW
 *	DESCRIPTION	:
 *
 *==history======================================================================
 *
 ********************************************************************************/
unsigned char clg3GetPush(unsigned char sw_data, unsigned char sts){
	return (unsigned char)(sts_push[sts] & sw_data);
}

/********************************************************************************
 *
 *	Function	:	unsigned char clg3GetDip(unsigned char sts)
 *	INPUT		:	sts		:	擾SW
 *	OUTPUT		:	result	:	SW
 *	DESCRIPTION	:
 *
 *==history======================================================================
 *
 ********************************************************************************/
unsigned char clg3GetDip(unsigned char sts){
	return (DIP_SW & sts);
}

/********************************************************************************
 *
 *	Function	:	unsigned char clg3SetDig(unsigned char where, unsigned char data, unsigned char sts)
 *	INPUT		:	where	:	Zbgӏ
 *					data	:	Zbgf[^
 *					sts		:	(TRUE:LEDp^[ FALSE:͕\)
 *	OUTPUT		:	FALSE	:	s
 *					TRUE	:	
 *	DESCRIPTION	:
 *
 *==history======================================================================
 *
 ********************************************************************************/
unsigned char clg3SetDig(unsigned char where, unsigned char data, unsigned char sts){
	/* auto variable */
	unsigned char result = TRUE;
	
	if(where < 4){
		if(sts == TRUE){
			dig_data[where] = CLG3IO_DIG_TBL[data];
		}
		else{
			dig_data[where] = data;
		}
	}
	else{
		result = FALSE;
	}
	
	return result;
}

/********************************************************************************
 *
 *	Function	:	unsigned char clg3InputIo(void)
 *	INPUT		:	None
 *	OUTPUT		:	FALSE	:	s
 *					TRUE	:	
 *	DESCRIPTION	:
 *
 *==history======================================================================
 *
 ********************************************************************************/
unsigned char clg3DispDig(void){
	/* auto variable */
	unsigned char result = TRUE;
	
	SEL_SEG = DIG_CLR_MSK;				/* \NA */
	DIG = dig_data[set_seg];			/* f[^Zbg */
	SEL_SEG = CLG3_DISP_TBL[set_seg];	/* \ */
	set_seg++;
	if(set_seg >= 4) set_seg = 0;
	
	return result;
}
